@@ -1,11 +1,12 @@
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import print_function
-
 import json
 import os
-from sys import stderr
+from sys import stderr,stdout
+from trident.backend.common import *
+from trident.backend.model import *
 
-from .common import *
 
 _session=get_session()
 _trident_dir=get_trident_dir()
@@ -26,21 +27,6 @@ def write_config(_config_path):
         # Except permission denied.
         pass
 
-#
-# # Attempt to read Trident config file.
-# _config_path = os.path.expanduser(os.path.join(_trident_dir, 'trident.json'))
-# if os.path.exists(_config_path):
-#     try:
-#         with open(_config_path) as f:
-#             _config = json.load(f)
-#     except ValueError:
-#         _config = {}
-#     for k,v in _config.items():
-#         if k=='floatx':
-#             assert v in {'float16', 'float32', 'float64'}
-#         _session[k]=v
-
-
 
 # Save config file, if possible.
 if not os.path.exists(_trident_dir):
@@ -59,50 +45,46 @@ def get_image_backend():
     return _session.image_backend
 
 
-
-
 # Set backend based on TRIDENT_BACKEND flag, if applicable.
 if 'TRIDENT_BACKEND' in os.environ:
     if _session.backend!=os.environ['TRIDENT_BACKEND']:
         _session.backend = os.environ['TRIDENT_BACKEND']
         write_config(_config_path)
+else:
+    try:
+        import torch
+        os.environ['TRIDENT_BACKEND']='pytorch'
+    except:
+        try:
+            import tensorflow
+            os.environ['TRIDENT_BACKEND'] = 'tensorflow'
+        except:
+            pass
 
 
-
-if _session.backend == 'cntk':
-    stderr.write('Using CNTK backend\n')
-    stderr.write('Image Data Format: channels_first.\n')
-    stderr.write('Image Channel Order: rgb.\n')
-    _session.backend = 'cntk'
-    _session.image_data_format = 'channels_first'
-    _session.image_channel_order = 'rgb'
-    from .cntk_backend import *
-    from .cntk_ops import *
-
-elif _session.backend == 'pytorch':
-    stderr.write('Using Pytorch backend.\n')
-    stderr.write('Image Data Format: channels_first.\n')
-    stderr.write('Image Channel Order: rgb.\n')
+if _session.backend == 'pytorch':
+    stdout.write('Using Pytorch backend.\n')
+    stdout.write('Image Data Format: channels_first.\n')
+    stdout.write('Image Channel Order: rgb.\n')
     _session.backend='pytorch'
     _session.image_data_format='channels_first'
     _session.image_channel_order='rgb'
 
-    from .pytorch_backend import *
-    from .pytorch_ops import *
+
+
     #module = importlib.import_module(mName)
     #layers=importlib.import_module('layers.pytorch_layers')
 
 
 elif _session.backend == 'tensorflow':
-    stderr.write('Using TensorFlow backend.\n')
-    stderr.write('Image Data Format: channels_last.\n')
-    stderr.write('Image Channel Order: rgb.\n')
+    stdout.write('Using TensorFlow backend.\n')
+    stdout.write('Image Data Format: channels_last.\n')
+    stdout.write('Image Channel Order: rgb.\n')
     _session.backend = 'tensorflow'
     _session.image_data_format = 'channels_last'
     _session.image_channel_order = 'rgb'
 
-    from .tensorflow_backend import  *
-    from  .tensorflow_ops import *
+
 
 if 'TRIDENT_IMG_BACKEND' in os.environ:
     _image_backend = os.environ['TRIDENT_IMG_BACKEND']
@@ -113,10 +95,12 @@ if 'TRIDENT_IMG_BACKEND' in os.environ:
 
 
 
+
+
 if _session.image_backend == 'opencv':
-    stderr.write('Using opencv image backend\n')
+    stdout.write('Using opencv image backend\n')
 elif _session.image_backend== 'pillow':
-    stderr.write('Using pillow image backend.\n')
+    stdout.write('Using pillow image backend.\n')
 
 
 
