@@ -47,7 +47,9 @@ begin_time = last_time
 
 
 class ModelBase(object):
-    def __init__(self, inputs=None, output=None, input_shape=None, name='', **kwargs):
+    def __init__(self, inputs=None,  input_shape=None,output=None, name='', **kwargs):
+        if isinstance(inputs,tuple) and isinstance(inputs[0],int):
+            input_shape.inputs=inputs,input_shape
         self.inputs = OrderedDict()
         self._outputs = OrderedDict()
         self._targets = OrderedDict()
@@ -78,6 +80,7 @@ class ModelBase(object):
         self.gradscaler = None
         self.grad_clipping_by_norm = False
         self.grad_clipping_threshold = None
+        self.use_output_as_loss = False
         self.training_context = {'losses': OrderedDict(),  # loss_wrapper
                                  'metrics': OrderedDict(),  # loss_wrapper
                                  'grads_state': OrderedDict(),  # loss_wrapper
@@ -572,10 +575,17 @@ class ModelBase(object):
                 if isinstance(output, (list, tuple)):
                     for i in range(len(output)):
                         train_data[self.outputs.key_list[i]] = output[i]
+
                 elif 'tensor' in output.__class__.__name__.lower():
                     train_data[self.outputs.key_list[0]] = output
                 else:
                     train_data[self.outputs.key_list[0]] = output
+                    if self.use_output_as_loss==True:
+                        if 'self.outputs.key_list[0]' not in self.training_context['losses']:
+                            self.training_context['losses'][self.outputs.key_list[0]] = []
+                        this_loss=output.sum()
+                        self.training_context['losses'][self.outputs.key_list[0]]=this_loss
+                        self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
             except Exception as e:
                 print(e)
                 PrintException()
@@ -683,15 +693,18 @@ class ModelBase(object):
                     elif is_tensor(self._model):
                         self.log_weight(weghts=self._model)
 
+
+
                 if test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
-                    output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
-                    if isinstance(output, (list, tuple)):
-                        for i in range(len(output)):
-                            test_data[self.outputs.key_list[i]] = output[i]
-                    elif 'tensor' in output.__class__.__name__.lower():
-                        test_data[self.outputs.key_list[0]] = output
+                    tmp_output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
+                    if isinstance(tmp_output, (list, tuple)):
+                        for i in range(len(tmp_output)):
+                            test_data[self.outputs.key_list[i]] = tmp_output[i]
+                    elif 'tensor' in tmp_output.__class__.__name__.lower():
+                        test_data[self.outputs.key_list[0]] = tmp_output
                     else:
-                        test_data[self.outputs.key_list[0]] = output
+                        test_data[self.outputs.key_list[0]] = tmp_output
+
 
 
 
