@@ -2,7 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from ..misc.ipython_utils import is_in_ipython, is_in_colab
+from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 
 if is_in_ipython():
     from IPython import display
@@ -18,7 +18,7 @@ import sys
 import cv2
 import numpy as np
 
-from .common import get_session,floatx
+from trident.backend.common import get_session,floatx
 
 
 
@@ -75,16 +75,15 @@ def save_mask(arr, file_path):
 
 
 def image2array(img):
-    '''
-
+    """
     Parameters
-    ----------
-    img  (string, pillow image or numpy.ndarray): Image to be converted to ndarray.
+        img  (string, pillow image or numpy.ndarray): Image to be converted to ndarray.
 
-    Returns ndarray  (HWC / RGB)
-    -------
+    Returns
+        ndarray  (HWC / RGB)
 
-    '''
+
+    """
     if isinstance(img,str):
         if os.path.exists(img) and img.split('.')[-1] in ('jpg','jpeg','png','bmp','tiff'):
             img=plt.imread(img)[::-1]
@@ -92,6 +91,7 @@ def image2array(img):
             return None
     arr=None
     if isinstance(img, np.ndarray):
+        arr=img
         if arr.ndim not in [2, 3]:
             raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
         if arr.ndim == 3:
@@ -108,16 +108,15 @@ def image2array(img):
         arr = np.ascontiguousarray(arr)
     return arr
 def array2image(arr:np.ndarray):
-    '''
-
+    """
     Parameters
-    ----------
-    arr  ndarry  : array need to convert back to image
+        arr  ndarry  : array need to convert back to image
 
-    Returns   pillow image
-    -------
+    Returns
+        pillow image
 
-    '''
+
+    """
     # confirm back to numpy
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
@@ -142,16 +141,13 @@ def array2image(arr:np.ndarray):
 
 
 def mask2array(img):
-    '''
+    """
+    Args
+        img  (string, pillow image or numpy.ndarray): Image to be converted to ndarray.
+    Returns
+        ndarray  (HW / single channel)
 
-    Parameters
-    ----------
-    img  (string, pillow image or numpy.ndarray): Image to be converted to ndarray.
-
-    Returns ndarray  (HW / single channel)
-    -------
-
-    '''
+    """
     arr = None
     if isinstance(img, str):
         if os.path.exists(img) and img.split('.')[-1] in ('jpg', 'jepg', 'png', 'bmp', 'tiff'):
@@ -170,27 +166,13 @@ def mask2array(img):
     return arr
 
 def array2mask(arr:np.ndarray):
-    '''
+    """
+    Args
+        arr  ndarry  : array need to convert back to image
 
-    Parameters
-    ----------
-    arr  ndarry  : array need to convert back to image
-
-    Returns   pillow image
-    -------
-
-    '''
-    # confirm back to numpy
-    '''
-
-    Parameters
-    ----------
-    img  (string, pillow image or numpy.ndarray): Image to be converted to ndarray.
-
-    Returns ndarray  (HWC / RGB)
-    -------
-
-    '''
+    Returns
+        pillow image
+    """
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
     arr=np.squeeze(arr)
