@@ -59,7 +59,7 @@ def read_image(im_path:str):
             img = plt.imread(im_path)
             return img[::-1]
         except Exception as e:
-            sys.stderr.write(e)
+            print(e)
             return None
 
 def read_mask(im_path:str):
@@ -84,7 +84,7 @@ def read_mask(im_path:str):
             else:
                 raise ValueError('extension {0} not support (jpg, jepg, png, bmp, tiff)'.format(im_path.split('.')[-1]))
     except Exception as e:
-        sys.stderr.write(e)
+        print(e)
         return None
 
 def save_image(arr,file_path):
