@@ -1,3 +1,4 @@
+"""data_loader: The ready-to-use data provider"""
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -6,12 +7,12 @@ import pickle
 
 import numpy as np
 
-from .data_provider import *
-from .dataset import *
-from .mask_common import *
-from .utils import *
-from ..backend.common import floatx
-from ..misc.ipython_utils import *
+from trident.data.data_provider import *
+from trident.data.dataset import *
+from trident.data.mask_common import *
+from trident.data.utils import *
+from trident.backend.common import floatx
+from trident.misc.ipython_utils import *
 
 try:
     from urllib.request import urlretrieve
@@ -25,6 +26,16 @@ _backend = _session.backend
 if 'TRIDENT_BACKEND' in os.environ:
     _backend = os.environ['TRIDENT_BACKEND']
 
+
+if _backend=='pytorch':
+    from trident.backend.pytorch_backend import *
+    from  trident.backend.pytorch_ops import *
+
+elif _backend == 'tensorflow':
+    from trident.backend.tensorflow_backend import *
+    from trident.backend.tensorflow_ops import *
+
+
 if not os.path.exists(_trident_dir):
     try:
         os.makedirs(_trident_dir)
@@ -34,77 +45,21 @@ if not os.path.exists(_trident_dir):
         pass
 
 
-def to_onehot(arr):
-    if isinstance(arr, list):
-        arr = np.array(arr)
-    elif not isinstance(arr, np.ndarray):
-        raise ValueError('You should input a list of integer or ndarray.')
-    items = np.unique(arr)
-    items = np.argsort(items)
-    if np.min(items) < 0:
-        raise ValueError('Negative value cannot convert to onhot.')
-    elif np.sum(np.abs(np.round(arr) - arr)) > 0:
-        raise ValueError('Only integer value can convert to onhot.')
-    else:
-        max_value = int(np.max(items))
 
-        output_shape = list(arr.shape)
-        output_shape.append(max_value + 1)
-        output = np.zeros(output_shape, dtype=floatx())
-        arr = arr.astype(np.uint8)
-        for i in range(max_value):
-            onehot = np.zeros(max_value + 1, dtype=floatx())
-            onehot[i] = 1
-            output[arr == i] = onehot
-        return output
-
-
-#
-# def download_image(image, temproot, imageroot, flog=None):
-#     # Check existing file.
-#     try:
-#         temppath, imagepath = (os.path.join(root, image['path']) for root in (temproot, imageroot))
-#
-#     except Exception as e:
-#         sys.stderr('Unexpected exception before attempting download of image {0}.'.format(e))
-#
-#
-#     # GET and save to temp location.
-#     try:
-#         r = requests.get(image['url'])
-#         if r.status_code == 200:
-#             ensure_parent_dir(temppath)
-#             with open(temppath, 'wb') as fout:
-#                 for chunk in r.iter_content(1024): fout.write(chunk)
-#             logmsg('Saved  {}.'.format(temppath), flog=flog)
-#         else:
-#             logmsg('Status code {} when requesting {}.'.format(r.status_code, image['url']))
-#             return DownloadResult.DOWNLOAD_FAILED
-#     except Exception as e:
-#         stderr('Unexpected exception when downloading image {!r}.'.format(image), e, flog=flog)
-#         return DownloadResult.DOWNLOAD_FAILED
-#     # Check contents.
-#     try:
-#         if check_image(image, temppath):
-#             stderr('Image contents look good.)
-#         else:
-#             stderr('Image contents are wrong.')
-#             return DownloadResult.MD5_FAILED
-#     except Exception as e:
-#         stderr('Unexpected exception when checking file contents for image {!r}.'.format(image), e)
-#         return DownloadResult.MYSTERY_FAILED
-#     # Move image to final location.
-#     try:
-#         ensure_parent_dir(imagepath)
-#         os.rename(temppath, imagepath)
-#     except Exception as e:
-#         stderr('Unexpected exception when moving file from {} to {} for image {!r}.'.format(temppath, imagepath,
-#         image), e, flog=flog)
-#         return DownloadResult.MYSTERY_FAILED
-#     return DownloadResult.NEW_OK
 
 
 def load_mnist(dataset_name='mnist', **kwargs):
+    """data loader for mnist data
+
+    Args:
+        dataset_name (string): if 'minist'  will return the traditional mnist data/ label,
+            if 'fashion-mnist' will return the fashion-mnist' data
+
+
+    Returns:
+        a tuple of data and label
+
+    """
     dataset_name = dataset_name.strip().lower().replace('minist', 'mnist')
 
     if dataset_name.lower() not in ['mnist', 'fashion-mnist']:
@@ -159,7 +114,7 @@ def load_mnist(dataset_name='mnist', **kwargs):
             else:
                 testData = Iterator(data=imagedata, label=labeldata)
 
-            dataset = DataProviderV2(dataset_name, traindata=trainData, testdata=testData)
+            dataset = DataProvider(dataset_name, traindata=trainData, testdata=testData)
             dataset.binding_class_names(
                 [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] if dataset_name == 'mnist' else ['T-shirt/top', 'Trouser', 'Pullover',
                                                                                 'Dress', 'Coat', 'Sandal', 'Shirt',
@@ -171,6 +126,17 @@ def load_mnist(dataset_name='mnist', **kwargs):
 
 
 def load_cifar(dataset_name='cifar10'):
+    """data loader for mnist data
+
+    Args:
+        dataset_name (string): if 'cifar10'  will return the traditional cifar10 data/ label,
+            if 'cifar100' will return the cifar100 data
+
+
+    Returns:
+        a tuple of data and label
+
+    """
     dataset_name = dataset_name.strip().lower().replace(' ', '')
 
     if dataset_name.lower() not in ['cifar10', 'cifar100']:
@@ -189,22 +155,36 @@ def load_cifar(dataset_name='cifar10'):
             pass
 
     """Load CIFAR data from `path`"""
+    _,filename,ext=split_path(baseURL)
+    download_file(baseURL, dirname, filename+ext, dataset_name)
+    file_path = os.path.join(dirname, filename+ext)
 
-    download_file(baseURL, dirname, baseURL.split('/')[-1].strip(), dataset_name)
-    file_path = os.path.join(dirname, baseURL.split('/')[-1].strip())
-    if '.tar' in file_path:
-        extract_archive(file_path, dirname, archive_format='tar')
-    extract_path = os.path.join(dirname, baseURL.split('/')[-1].strip().split('.')[0])
+
+    if '.tar' in ext:
+        extract_archive(file_path, dirname, archive_format='auto')
+    filelist = glob.glob(dirname + '/*/*.*')
+    extract_path ,_,_= split_path(filelist[0])
     filelist = [f for f in os.listdir(extract_path) if os.path.isfile(os.path.join(extract_path, f))]
+    data=[]
+    label=[]
+    test_data=[]
+    test_label=[]
+    for file_path in filelist:
+        if 'data_batch' in file_path:
+            with open(os.path.join(extract_path,file_path), 'rb') as f:
+                entry = pickle.load(f, encoding='latin1')
+                data.extend(np.reshape(entry['data'],(-1,32, 32, 3)))
+                label.extend(entry['labels'])
+        elif 'test_batch' in file_path:
+            with open(os.path.join(extract_path,file_path), 'rb') as f:
+                entry = pickle.load(f, encoding='latin1')
+                test_data.extend(np.reshape(entry['data'],(-1,32, 32, 3)))
+                test_label.extend(entry['labels'])
 
-    data, labels = open_pickle(os.path.join(extract_path, 'train'), 'data', 'fine_labels')
-    testdata, testlabels = open_pickle(os.path.join(extract_path, 'test'), 'data', 'fine_labels')
-    data = data.reshape(data.shape[0], 32, 32, 3).astype(_session.floatx)
-    testdata = testdata.reshape(testdata.shape[0], 32, 32, 3).astype(_session.floatx)
 
-    trainData = Iterator(data=ImageDataset(data), label=LabelDataset(labels))
-    testData = Iterator(data=ImageDataset(testdata), label=LabelDataset(testlabels))
-    dataset = DataProviderV2(dataset_name, traindata=trainData, testdata=testData)
+    trainData = Iterator(data=ImageDataset(data), label=LabelDataset(label))
+    testData = Iterator(data=ImageDataset(test_data), label=LabelDataset(test_label))
+    dataset = DataProvider(dataset_name, traindata=trainData, testdata=testData)
     dataset.binding_class_names(['airplane', 'automobile', 'bird', 'cat', 'deer', 'dog', 'frog', 'horse', 'ship',
                                  'truck'] if dataset_name == 'cifar10' else [], 'en-US')
     return dataset
@@ -314,6 +294,8 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
             for i in range(len(class_names)):
                 class_name = class_names[i]
                 class_imgs = glob.glob(base_folder + '/{0}/*.*g'.format(class_name))
+                if len(class_imgs)==0:
+                    class_imgs = glob.glob(base_folder + '/{0}/*/*.*g'.format(class_name))
                 print(base_folder + '/{0}/*.*g'.format(class_name))
                 print(len(class_imgs))
                 labels.extend([i] * len(class_imgs))
@@ -325,14 +307,14 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
             labelsdata.binding_class_names(class_names)
 
             traindata = Iterator(data=imagedata, label=labelsdata)
-            dataset = DataProviderV2(dataset_name, traindata=traindata)
+            dataset = DataProvider(dataset_name, traindata=traindata)
             dataset.binding_class_names(class_names)
 
         else:
             imgs = glob.glob(base_folder + '/*.*g')
             imagedata = ImageDataset(imgs, expect_data_type=expect_data_type, get_image_mode=GetImageMode.processed)
             traindata = Iterator(data=imagedata)
-            dataset = DataProviderV2(dataset_name, traindata=traindata)
+            dataset = DataProvider(dataset_name, traindata=traindata)
         return dataset
     else:
         raise ValueError('')
@@ -392,12 +374,22 @@ def load_stanford_cars(dataset_name='cars', kind='train', is_flatten=None, is_on
     return dataset
 
 
-def load_lfw(kind='train', is_flatten=None, is_onehot=None):
-    dataset_name = 'lfw'
-    kind = kind.strip().lower().replace('ing', '')
-    if _backend in ['tensorflow', 'cntk'] and is_onehot is None:
-        is_onehot = True
+def load_lfw(format='aligned_face', is_paired=False):
+    """
 
+    Args:
+        format (str):
+        is_paired (bool): if True, will return  anchor-positive-negative pair, or return image-classification label.
+
+    Returns:
+
+    Exsamples:
+        >>> dataloader=load_lfw(only_face_area=True, only_aligned_face=True)
+        >>> len(dataloader)
+        10
+
+    """
+    dataset_name = 'lfw'
     dirname = os.path.join(_trident_dir, dataset_name)
     if not os.path.exists(dirname):
         try:
@@ -407,24 +399,126 @@ def load_lfw(kind='train', is_flatten=None, is_onehot=None):
             # in multi-threaded environments.
             pass
 
-    download_file('http://vis-www.cs.umass.edu/lfw/lfw.tgz', dirname, 'lfw.tgz')
+    tar_file_path = os.path.join(dirname, 'lfw.tgz')
+    if format=='aligned_face':
+        download_file_from_google_drive('1sVbU8NHC7kzDkqByTtPiib9_EY23eeW7', dirname, 'lfw-crop.tar')
+    elif format=='raw':
+        download_file('http://vis-www.cs.umass.edu/lfw/lfw.tgz', dirname, 'lfw.tgz')
+
     download_file('http://vis-www.cs.umass.edu/lfw/pairsDevTrain.txt', dirname, 'pairsDevTrain.txt')
     download_file('http://vis-www.cs.umass.edu/lfw/pairsDevTest.txt', dirname, 'pairsDevTest.txt')
 
+    # if only_face_area:
+    #     if only_aligned_face:
     tar_file_path = os.path.join(dirname, 'lfw.tgz')
-    extract_path = os.path.join(_trident_dir, 'lfw')
     extract_archive(tar_file_path, dirname, archive_format='tar')
-    dataset = load_folder_images(dataset_name, dirname)
+    extract_path = os.path.join(dirname,'lfw-crop' if format=='aligned_face' else 'lfw')
+    # if _backend=='tensorflow':
+    #     import trident.models.tensorflow_mtcnn as mtcnn
+    # else:
+    #     import trident.models.pytorch_mtcnn as mtcnn
+    #
+    # detector = mtcnn.Mtcnn(pretrained=True,verbose=False)
+    # detector.minsize = 70
+    #
+    # detector.detection_threshould = [0.8, 0.8, 0.9]
+    # detector.nms_threshould = [0.5, 0.5, 0.3]
+    # faces = glob.glob(os.path.join(dirname, 'lfw-deepfunneled' if only_aligned_face else 'lfw') + '/*/*.*g')
+    # faces_crop=glob.glob(os.path.join(dirname, 'lfw-crop')+ '/*/*.*g')
+    # if len(faces_crop)==0 or len(faces_crop)!=len(faces):
+    #     for  m in range(len(faces)):
+    #         img_path=faces[m]
+    #         new_path = img_path.replace(os.path.join(dirname, 'lfw-deepfunneled' if only_aligned_face else 'lfw'), os.path.join(dirname, 'lfw-crop'))
+    #
+    #         if os.path.exists(new_path) and image2array(new_path) is not None:
+    #             pass
+    #         else:
+    #             make_dir_if_need(new_path)
+    #             img = read_image(img_path)
+    #             detector.detection_threshould = [0.8, 0.8, 0.9]
+    #             results = detector.infer_single_image(img_path)
+    #             if len(results)==0 or results is None:
+    #                 detector.detection_threshould = [0.5, 0.7, 0.8]
+    #                 results = detector.infer_single_image(img_path)
+    #
+    #             results = detector.rerec(to_tensor(results), img.shape)
+    #             results[:, 0] = clip(results[:, 0], 0, img.shape[1])
+    #             results[:, 1] = clip(results[:, 1], 0, img.shape[0])
+    #             results[:, 2] = clip(results[:, 2], 0, img.shape[1])
+    #             results[:, 3] = clip(results[:, 3], 0, img.shape[0])
+    #             area=detector.area_of(results[:,:2],results[:,2:4])
+    #             area_mask=area>500
+    #             results=to_numpy(results[area_mask,:])
+    #
+    #             if len(results)>1:
+    #                 area=area[area_mask]
+    #                 idx=argmax(area).item()
+    #                 results=results[idx:idx+1]
+    #             if results is not None and len(results)>0:
+    #                 for k in range(len(results)):
+    #                     result =np.round(results[k][:4]).astype(np.uint8)
+    #                     x1, y1, x2, y2 = result[0],result[1],result[2],result[3]
+    #                     crop_img =np.clip( img.copy()[y1:y2,x1:x2, :],0,255).astype(np.uint8)
+    #
+    #                     array2image(crop_img).save(new_path)
+    #             else:
+    #
+    #                 print(img_path+' get {0} results!'.format(results))
+    #
+    #         sys.stdout.write('\r {0}/{1} image processed!'.format(m,len(faces)))
+    #         sys.stdout.flush()
+    #
+    #
+    data_provider = load_folder_images(dataset_name, os.path.join(dirname, 'lfw-crop'))
 
-    extract_archive(tar_file_path, dirname, archive_format='tar')
-    dataset = load_folder_images(dataset_name, dirname)
-    return dataset
+    if is_paired:
+        storage=OrderedDict()
+        data=list(data_provider.traindata.data)
+        label=list(data_provider.traindata.label)
+        class_names=data_provider._class_names
+        for i in range(len(data)):
+            class_label=class_names[label[i]]
+            if class_label not in storage:
+                storage[class_label] = []
+            storage[class_label].append(data[i])
+
+        metric=MetricDataset(storage.item_list)
+        data_provider = DataProvider(dataset_name, data=metric, scenario='train')
+
+
+    # extract_archive(tar_file_path, dirname, archive_format='tar')
+    # dataset = load_folder_images(dataset_name, dirname)
+    return data_provider
 
 
 def load_examples_data(dataset_name):
+    """data loader for AllanYiin deep learning course exsample data
+
+    Args:
+        dataset_name (string):
+            'pokemon': pokemon images for autoencoder
+            hanzi': Chinese hanzi hand writing recognition data
+            'animals': Challenging animals recognition data
+            'nsfw': porn detection data
+            'simpsons': simpson images for gan
+            'horse2zebra' : horse2zebra cyclegan training data
+            'people': Supervisely human segmentation data
+            autodrive':Streetview segmentation data
+            'superresolution': Collections of high resolution images for superresolution model
+            'anpr': Automatic number-plate recognition data
+            'beauty':beauty detection data
+
+    Returns:
+        a tuple of data and label
+
+    References
+        If you want know more how to use these data to build model, please go to my github repository:
+        https://github.com/AllanYiin/DeepBelief_Course5_Examples
+
+    """
     dataset_name = dataset_name.strip().lower()
     if dataset_name.lower() not in ['pokemon', 'hanzi', 'animals', 'nsfw', 'simpsons', 'horse2zebra', 'people',
-                                    'autodrive', 'superresolution', 'anpr', 'beauty']:
+                                    'autodrive', 'superresolution', 'anpr', 'beauty','antisproofing','facelandmarks']:
         raise ValueError('Not a  valid  dataset_name.')
     dataset_name = 'examples_' + dataset_name
     dirname = os.path.join(_trident_dir, dataset_name)
@@ -486,7 +580,7 @@ def load_examples_data(dataset_name):
         test_iter = Iterator(data=testarray, label=testlabel)
         print('training images: {0}  test images:{1}'.format(len(trainarray), len(testarray)))
 
-        dataset = DataProviderV2(dataset_name, traindata=train_iter, testdata=test_iter)
+        dataset = DataProvider(dataset_name, traindata=train_iter, testdata=test_iter)
         dataset.binding_class_names(['drawing', 'hentai', 'neutral', 'porn', 'sexy'], 'en-us')
         dataset.binding_class_names(['繪畫', '色情漫畫', '中性', '色情', '性感'], 'zh-tw')
         dataset.binding_class_names(['绘画', '色情漫画', '中性', '色情', '性感'], 'zh-cn')
@@ -516,7 +610,7 @@ def load_examples_data(dataset_name):
                              get_image_mode=GetImageMode.processed)
         train_iter = Iterator(data=trainA, unpair=trainB)
         test_iter = Iterator(data=testA, unpair=testB)
-        dataset = DataProviderV2(dataset_name, traindata=train_iter, testdata=test_iter)
+        dataset = DataProvider(dataset_name, traindata=train_iter, testdata=test_iter)
         print('get horse2zebra images :{0}'.format(len(dataset)))
         return dataset
     elif dataset_name == 'examples_people':
@@ -534,7 +628,7 @@ def load_examples_data(dataset_name):
 
         imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb)
         mskdata = MaskDataset(masks=masks, expect_data_type=ExpectDataType.binary_mask)
-        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
+        dataset = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get people images :{0}'.format(len(dataset)))
         return dataset
     elif dataset_name == 'examples_autodrive':
@@ -558,7 +652,7 @@ def load_examples_data(dataset_name):
         for i in range(len(label_codes)):
             mskdata.palette[label_names[i]] = label_codes[i]
 
-        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
+        dataset = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get autodrive images :{0}'.format(len(dataset)))
         return dataset
     elif dataset_name == 'examples_superresolution':
@@ -568,9 +662,10 @@ def load_examples_data(dataset_name):
         imgs = glob.glob(os.path.join(dirname, '*.*g'))
         imgs.extend(glob.glob(os.path.join(dirname, '*.bmp')))
         print('get super resolution images :{0}'.format(len(imgs)))
+
         imgdata = ImageDataset(images=imgs * 2, expect_data_type=ExpectDataType.rgb, symbol='lr')
         labeldata = ImageDataset(images=imgs * 2, expect_data_type=ExpectDataType.rgb, symbol='hr')
-        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
+        dataset = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
         return dataset
     elif dataset_name == 'examples_beauty':
         download_file_from_google_drive('1aJhxN9IqsxuayhRTm-gmxk6PiLe5wm9X', dirname, 'beauty.tar')
@@ -598,9 +693,49 @@ def load_examples_data(dataset_name):
         print('{0} faces loaded...'.format(len(imgs)))
         imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb, symbol='faces')
         labeldata = NumpyDataset(data=ratings, expect_data_type=ExpectDataType.array_data, symbol='ratings')
-        data_provider = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
+        data_provider = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
         return data_provider
 
+    elif dataset_name == 'examples_facelandmarks':
+        download_file_from_google_drive('1aJhxN9IqsxuayhRTm-gmxk6PiLe5wm9X', dirname, 'beauty.tar')
+        tar_file_path = os.path.join(dirname, 'beauty.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        # 讀取圖片數據
+        images_dict = {}
+        with open(os.path.join(dirname, 'images_dict.pkl'), 'rb') as fp:
+            images_dict = pickle.load(fp)
+
+        f = open(os.path.join(dirname, 'All_Ratings.txt'), encoding='utf-8-sig').readlines()
+        imgs = []
+        landmark = []
+        ratings = []
+        for row in f:
+            data = row.strip().split('\t')
+            if 'images\\' + data[0] in images_dict:
+                img = images_dict['images\\' + data[0]][0]
+                img = img.transpose([2, 0, 1])[::-1].transpose([1, 2, 0])
+                imgs.append(img)
+                landmark = images_dict['images\\' + data[0]][1].astype(np.float32) / 256.0
+                rating =np.zeros(2)
+                if 'm' in  data[0]:
+                    rating[0]=1
+                if 'w' in  data[0]:
+                    rating[1]=1
+
+                rating = np.concatenate([landmark.reshape(-1), rating], axis=0).astype(np.float32)
+                ratings.append(rating)
+        print('{0} faces loaded...'.format(len(imgs)))
+        imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb, symbol='faces')
+        labeldata = NumpyDataset(data=ratings, expect_data_type=ExpectDataType.array_data, symbol='ratings')
+        data_provider = DataProvider(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=labeldata))
+        return data_provider
+    elif dataset_name == 'examples_antisproofing':
+        download_file_from_google_drive('1e7Zjn2MHNCvA5gXdJUECzY8NjK4KVpa7', dirname, 'antisproofing.tar')
+        tar_file_path = os.path.join(dirname, 'antisproofing.tar')
+        make_dir_if_need(os.path.join(dirname, 'antisproofing'))
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        data_provider = load_folder_images(dataset_name,os.path.join(dirname, 'antisproofing'))
+        return data_provider
     elif dataset_name == 'examples_anpr':
         download_file_from_google_drive('1uGBd8tXlP0TZAXNgrR6H0jl5MXj7VPbN', dirname, 'anpr.tar')
         tar_file_path = os.path.join(dirname, 'anpr.tar')
