@@ -10,9 +10,9 @@ import warnings
 
 import numpy as np
 
-from .image_common import check_same_size
-from ..backend.common import OrderedDict
-from ..backend.load_backend import get_backend
+from trident.data.image_common import check_same_size
+from trident.backend.common import OrderedDict
+from trident.backend.load_backend import get_backend
 
 __all__ = ['Sampler','SequentialSampler','RandomSampler','BatchSampler']
 
@@ -37,7 +37,7 @@ class Sampler(object):
 class SequentialSampler(Sampler):
     r"""Samples elements sequentially, always in the same order.
 
-    Arguments:
+    Args:
         data_source (Dataset): dataset to sample from
     """
 
@@ -55,7 +55,7 @@ class RandomSampler(Sampler):
     r"""Samples elements randomly. If without replacement, then sample from a shuffled dataset.
     If with replacement, then user can specify ``num_samples`` to draw.
 
-    Arguments:
+    Args:
         data_source (Dataset): dataset to sample from
         num_samples (int): number of samples to draw, default=len(dataset)
         replacement (bool): samples are drawn with replacement if ``True``, default=False
@@ -100,7 +100,7 @@ class BatchSampler(Sampler):
         drop_last (bool): If ``True``, the sampler will drop the last batch if
             its size would be less than ``batch_size``
 
-    Example:
+    Examples:
         >>> list(BatchSampler(SequentialSampler(range(10)), batch_size=3, drop_last=False))
         [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
         >>> list(BatchSampler(SequentialSampler(range(10)), batch_size=3, drop_last=True))
