@@ -2,7 +2,7 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import builtins
 import inspect
 import math
 from collections import OrderedDict
@@ -22,6 +22,7 @@ from torch.nn import init
 from torch.nn.parameter import Parameter
 
 from trident.backend.common import *
+from trident.backend.pytorch_ops import *
 from trident.backend.pytorch_backend import Layer, Sequential
 
 __all__ = [ 'MaxPool1d','MaxPool2d', 'MaxPool3d', 'MaxUnpool1d', 'MaxUnpool2d', 'MaxUnpool3d', 'AvgPool1d', 'AvgPool2d',
@@ -209,7 +210,7 @@ class MaxPool2d(_PoolNd):
             kh, kw = self.kernel_size[-2:]
             sh, sw = self.strides[-2:]
 
-            oh, ow = math.ceil(ih // sh), math.ceil(iw // sw)
+            oh, ow =ceil(true_divide(ih , sh)),ceil(true_divide(iw , sw))
             pad_h = max((oh - 1) * sh + (kh - 1) + 1 - ih, 0)
             pad_w = max((ow - 1) * sw + (kw - 1) + 1 - iw, 0)
             if pad_h % 2 == 1 and sh > 1:
@@ -640,7 +641,7 @@ class AvgPool2d(_PoolNd):
             kh, kw = self.kernel_size[-2:]
             sh, sw = self.strides[-2:]
 
-            oh, ow = math.ceil(ih / sh), math.ceil(iw / sw)
+            oh, ow = ceil(true_divide(ih,sh)), ceil(true_divide(iw, sw))
             pad_h = max((oh - 1) * sh + (kh - 1) + 1 - ih, 0)
             pad_w = max((ow - 1) * sw + (kw - 1) + 1 - iw, 0)
             if pad_h % 2 == 1 and sh > 1:
@@ -652,7 +653,7 @@ class AvgPool2d(_PoolNd):
             pad_h = self.padding[0] * 2
             pad_w = self.padding[1] * 2
 
-        self.padding = (int(pad_h / 2), int(pad_w / 2))
+        self.padding = (int(pad_h // 2), int(pad_w // 2))
 
     def forward(self, *x):
         x = enforce_singleton(x)
