@@ -69,11 +69,11 @@ def efficient_block(expand_ratio=1, filters_in=32, filters_out=16, kernel_size=3
 
         bottleneck = Sequential(
             DepthwiseConv2d_Block((kernel_size, kernel_size), depth_multiplier=1, strides=strides, auto_pad=True,
-                                  padding_mode='zero', normalization='batch', activation='swish', name=name + 'dwconv'),
+                                  padding_mode='zero', normalization='batch', activation='swish'),
             SqueezeExcite(se_filters=max(1, int(filters_in * se_ratio)), num_filters=filters_in,
                           use_bias=True) if 0 < se_ratio <= 1 else Identity(),
             Conv2d_Block((1, 1), num_filters=filters_out, strides=1, auto_pad=True, normalization='batch',
-                         activation=None, name=name + 'se'),
+                         activation=None),
             Dropout(dropout_rate=drop_rate) if is_shortcut and drop_rate > 0 else Identity())
 
         if is_shortcut:
@@ -84,13 +84,13 @@ def efficient_block(expand_ratio=1, filters_in=32, filters_out=16, kernel_size=3
     else:
         bottleneck = Sequential(
             Conv2d_Block((1, 1), num_filters=filters, strides=1, auto_pad=True, normalization='batch',
-                         activation='swish', name=name + 'expand_bn'),
+                         activation='swish'),
             DepthwiseConv2d_Block((kernel_size, kernel_size), depth_multiplier=1, strides=strides, auto_pad=True,
-                                  padding_mode='zero', normalization='batch', activation='swish', name=name + 'dwconv'),
+                                  padding_mode='zero', normalization='batch', activation='swish'),
             SqueezeExcite(se_filters=builtins.max(1, int(filters_in * se_ratio)), num_filters=filters,
                           use_bias=True) if 0 < se_ratio <= 1 else Identity(),
             Conv2d_Block((1, 1), num_filters=filters_out, strides=1, auto_pad=True, normalization='batch',
-                         activation=None, name=name + 'se'),
+                         activation=None),
             Dropout(dropout_rate=drop_rate) if is_shortcut and drop_rate > 0 else Identity())
         if is_shortcut:
             return ShortCut2d(Identity(), bottleneck)
