@@ -95,14 +95,14 @@ def MobileNet( input_shape=(3, 224, 224), classes=1000, use_bias=False, width_mu
     input_filters = _make_divisible(input_filters * width_mult, round_nearest)
     last_filters = _make_divisible(last_filters * max(1.0, width_mult), round_nearest)
     features = []
-    features.append(Conv2d_Block((3,3),num_filters=input_filters,strides=2,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6',name='first_layer'))
+    features.append(Conv2d_Block((3,3),num_filters=input_filters,strides=2,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6'))
     for t, c, n, s in inverted_residual_setting:
         output_filters = _make_divisible(c * width_mult, round_nearest)
         for i in range(n):
             strides = s if i == 0 else 1
             features.append( inverted_residual(input_filters,num_filters=output_filters, strides=strides, expansion=t,name='irb_{0}'.format(i)))
             input_filters = output_filters
-    features.append(Conv2d_Block((1,1), last_filters,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6',name='last_layer'))
+    features.append(Conv2d_Block((1,1), last_filters,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu6'))
     mobilenet.add_module('features',Sequential(*features,name='features'))
     mobilenet.add_module('gap',GlobalAvgPool2d())
     if include_top:
