@@ -699,8 +699,8 @@ class Ssd(Layer):
 
 
 class SsdDetectionModel(ImageDetectionModel):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(SsdDetectionModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(SsdDetectionModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
         self.detection_threshold = 0.6
         self.iou_threshold = 0.3
