@@ -79,8 +79,8 @@ def make_deterministic(seed: int = 19260817, cudnn_deterministic: bool = False):
 
 
 class Model(ModelBase):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(Model, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(Model, self).__init__(inputs, input_shape,output)
 
 
     def _initial_graph(self, inputs=None, output=None, input_shape=None,initializer=None):
@@ -104,12 +104,12 @@ class Model(ModelBase):
             else:
                 for m in range(len(inputs)):
                     inp=inputs[m]
-                    if is_tensor(inp):
+                    if is_tensor(inp) or isinstance(inp,np.ndarray):
                         input_name = 'input_{0}'.format(m)
                         self.inputs[input_name] = list(int_shape(inp))[1:]
         elif isinstance(inputs, dict):
             for k, v in inputs.items():
-                if is_tensor(v):
+                if is_tensor(v)or isinstance(v,np.ndarray):
                     self.inputs[k] =  list(int_shape(v))[1:]
         elif is_tensor(inputs):
             self.inputs['input'] = list(int_shape(inputs))[1:]
@@ -133,13 +133,24 @@ class Model(ModelBase):
                 self._model.signature.outputs = copy.deepcopy(self._outputs)
                 self._signature = self._model.signature
             else:
-                output.input_shape = input_shape
+                out=None
+                if inputs is not None:
+                    args=None
+                    if isinstance(inputs,dict):
+                        out = output(*list(inputs.values()))
+                    elif isinstance(inputs,(list,tuple)):
+                        out = output(*inputs)
+                    else:
+                        out = output(inputs)
 
-                dummay_input = to_tensor(np.random.standard_normal((1,)+tuple(input_shape)).astype(np.float32)).to(get_device())
-                # prevent pytorch 'ValueError: Expected more than 1 value per channel when training, got input size ....
-                output.to(get_device())
-                output.eval()
-                out = output(dummay_input)
+                else:
+
+                    output.input_shape = input_shape
+                    dummay_input = to_tensor(np.random.standard_normal((1,)+tuple(input_shape)).astype(np.float32)).to(get_device())
+                    # prevent pytorch 'ValueError: Expected more than 1 value per channel when training, got input size ....
+                    output.to(get_device())
+                    output.eval()
+                    out = output(dummay_input)
                 self._model = output
                 if isinstance(out, torch.Tensor):
                     self._outputs['output'] = to_list(out.size())[1:]
@@ -264,16 +275,20 @@ class Model(ModelBase):
             alias = loss.__name__
 
         if isinstance(loss, str):
-            loss_class = get_loss(loss)
-            alias = loss if loss_class is not None else alias
-            if  alias in self._losses:
-                dup_keys=[key for  key in self._losses.key_list if alias+'_' in key]
-                alias = alias + '_' + str(len(dup_keys)+1)
-            self._losses[alias] = loss_class(**kwargs) if len(kwargs)>0 else loss_class()
-            if hasattr(loss, 'forward'):
-                argnames = get_signature(self._losses[alias].forward,alias)
+            if loss=='output':
+                self.use_output_as_loss = True
+                return self
             else:
-                argnames = get_signature(self._losses[alias].__call__,alias)
+                loss_class = get_loss(loss)
+                alias = loss if loss_class is not None else alias
+                if  alias in self._losses:
+                    dup_keys=[key for  key in self._losses.key_list if alias+'_' in key]
+                    alias = alias + '_' + str(len(dup_keys)+1)
+                self._losses[alias] = loss_class(**kwargs) if len(kwargs)>0 else loss_class()
+                if hasattr(loss, 'forward'):
+                    argnames = get_signature(self._losses[alias].forward,alias)
+                else:
+                    argnames = get_signature(self._losses[alias].__call__,alias)
         elif inspect.isclass(loss) and inspect._is_type(loss):
             alias = loss.__class__.__name__ if alias is None or len(alias) == 0 else alias
             if alias in self._losses:
@@ -294,7 +309,7 @@ class Model(ModelBase):
             if hasattr(loss, 'forward'):
                 argnames = get_signature(self._losses[alias].forward,alias)
             else:
-                argnames = get_signature(self._losses[alias].__call__,alias)
+                argnames = get_signature(self._losses[alias],alias)
         elif inspect.isfunction(loss):
             if alias in self._losses:
                 dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
@@ -310,16 +325,23 @@ class Model(ModelBase):
         if hasattr(self._losses[alias],'signature') and self._losses[alias].signature is not None:
             pass
         else:
-            self._losses[alias].signature=argnames
-        self._losses[alias].signature.name=alias
-        if (len(self._losses[alias].signature.outputs) == 1 and self._losses[alias].signature.outputs.value_list[0] is None) or len(self._losses[alias].signature.outputs) == 0 :
-            self._losses[alias].signature.outputs = OrderedDict()
-            self._losses[alias].signature.outputs[alias] = None
-        if hasattr(self._losses[alias],'is_logsoftmax'):
-            if isinstance(self._model,Layer):
-                last_module=list(self._model.modules())[-1]
-                if isinstance(last_module,SoftMax):
-                    self._losses[alias].is_logsoftmax=True
+            try:
+                self._losses[alias].signature=argnames
+                self._losses[alias].signature.name = alias
+                if (len(self._losses[alias].signature.outputs) == 1 and self._losses[alias].signature.outputs.value_list[0] is None) or len(
+                        self._losses[alias].signature.outputs) == 0:
+                    self._losses[alias].signature.outputs = OrderedDict()
+                    self._losses[alias].signature.outputs[alias] = None
+                if hasattr(self._losses[alias], 'is_logsoftmax'):
+                    if isinstance(self._model, Layer):
+                        last_module = list(self._model.modules())[-1]
+                        if isinstance(last_module, SoftMax):
+                            self._losses[alias].is_logsoftmax = True
+                print(self._losses[alias].signature)
+            except:
+                print(argnames)
+
+
         # if isinstance(self._losses[alias], Layer) and len(list(self._losses[alias].parameters()))>0:
         #     para_dict={'params': list(self._losses[alias].parameters())}
         #     for k,v in self.optimizer.param_groups[0].items():
@@ -328,7 +350,7 @@ class Model(ModelBase):
         #
         #     self.optimizer.param_groups.append(para_dict)
         #     print('add loss parameters to optimizer ')
-        print(self._losses[alias].signature)
+
 
         self.loss_weights[alias] = float(loss_weight)
 
@@ -594,8 +616,7 @@ class Model(ModelBase):
                     # check input
                     for arg in self._model.signature.inputs.key_list:
                         data_feed[arg] = ''
-                    for arg in self._model.signature.inputs.key_list:
-                        if len(train_data) == 1:
+                        if len(train_data) == 1 and len(self._model.signature.inputs.key_list)==1:
                             data_feed[arg] = train_data.key_list[0]
                             available_fields.remove(train_data.key_list[0])
                         elif arg in available_fields:
@@ -622,43 +643,47 @@ class Model(ModelBase):
                                 'input argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
                                     arg))
 
-                    if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
-                        self.training_context['data_feed'] = data_feed
+
 
                     # check for target
-                    for i in range(len(self.targets)):
-                        arg = self.targets.key_list[i]
-                        data_feed[arg] = ''
-                        if len(train_data) == 1:
-                            data_feed[self.targets.key_list[0]] = train_data.key_list[0]
-                        elif arg in available_fields:
-                            data_feed[arg] = arg
-                            available_fields.remove(arg)
-                        elif arg == 'target' and 'label' in available_fields:
-                            data_feed[arg] = 'label'
-                            available_fields.remove('label')
-                        elif arg == 'target' and len(available_fields) == 1:
-                            data_feed[arg] = available_fields[0]
-                            available_fields.remove(available_fields[0])
-                        elif len(available_fields) > 0:
-                            target_shape = outshapes[i]
-                            for item in available_fields:
-                                data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
-                                if target_shape == data_shape:
-                                    data_feed[arg] = item
-                                    available_fields.remove(item)
-                                elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
-                                        train_data[item].dtype)) and target_shape[:-1] == data_shape:
-                                    data_feed[arg] = item
-                                    available_fields.remove(item)
-                                else:
-                                    Warning(
-                                        'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
-                                            arg))
-                    if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
-                        self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
+                    if len(available_fields)>0:
+                        for i in range(len(self.targets)):
+                            arg = self.targets.key_list[i]
+                            data_feed[arg] = ''
+                            if len(train_data) == 1:
+                                data_feed[self.targets.key_list[0]] = train_data.key_list[0]
+                            elif arg in available_fields:
+                                data_feed[arg] = arg
+                                available_fields.remove(arg)
+                            elif arg == 'target' and 'label' in available_fields:
+                                data_feed[arg] = 'label'
+                                available_fields.remove('label')
+                            elif arg == 'target' and len(available_fields) == 1:
+                                data_feed[arg] = available_fields[0]
+                                available_fields.remove(available_fields[0])
+                            elif len(available_fields) > 0:
+                                target_shape = outshapes[i]
+                                for item in available_fields:
+                                    data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
+                                    if target_shape == data_shape:
+                                        data_feed[arg] = item
+                                        available_fields.remove(item)
+                                    elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
+                                            train_data[item].dtype)) and target_shape[:-1] == data_shape:
+                                        data_feed[arg] = item
+                                        available_fields.remove(item)
+                                    else:
+                                        Warning(
+                                            'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
+                                                arg))
+                        if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
+                            self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
 
+                    # if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
+                    #     self.training_context['data_feed'] = data_feed
+                    # elif '' not in data_feed.value_list:
                     self.training_context['data_feed'] = data_feed
+
                     print('data_feed', data_feed)
             except:
                 PrintException()
@@ -910,8 +935,8 @@ class Model(ModelBase):
                 if is_tensor(value) and any_abnormal_number(value):
                     has_abnormal=True
                     where(is_nan(value).all(), random_normal_like(value, mean=0, std=0.02).to(get_device()).cast(value.dtype), value)
-
-            sys.stderr.write(self._model._name+ '  has_abnormal detected and  fixed!!\n')
+            if has_abnormal:
+                sys.stderr.write(self._model._name+ '  has_abnormal detected and  fixed!!\n')
             self._model.load_state_dict(pretrained_dict, strict=False)
             print('Model loaded!')
 
@@ -992,8 +1017,8 @@ class Model(ModelBase):
 
 
 class ImageClassificationModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageClassificationModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageClassificationModel, self).__init__(inputs, input_shape,output)
 
         self._class_names = []
         self._idx2lab = {}
@@ -1062,8 +1087,8 @@ class ImageClassificationModel(Model):
 
 
 class ImageDetectionModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageDetectionModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageDetectionModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
         self.detection_threshould = 0.5
 
@@ -1108,14 +1133,14 @@ class ImageDetectionModel(Model):
 
 
 class ImageSegmentationModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageSegmentationModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageSegmentationModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
 
 
 class ImageGenerationModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageGenerationModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageGenerationModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
 
     @property
@@ -1150,8 +1175,8 @@ class ImageGenerationModel(Model):
 
 
 class FaceRecognitionModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(FaceRecognitionModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(FaceRecognitionModel, self).__init__(inputs, input_shape,output)
 
         self._class_names = []
         self.preprocess_flow = []
@@ -1209,8 +1234,8 @@ class FaceRecognitionModel(Model):
 
 
 class LanguageModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(LanguageModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(LanguageModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
 
 
