@@ -58,10 +58,10 @@ def _to_tuple(x):
 
 
 class Model(ModelBase):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(Model, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None, input_shape=None,output=None):
+        super(Model, self).__init__(inputs, input_shape,output)
 
-    def _initial_graph(self, inputs=None, output=None, input_shape=None):
+    def _initial_graph(self, inputs=None,  input_shape=None,output=None):
         input_var = inputs
         out_var = output
         if output is None:
@@ -216,17 +216,20 @@ class Model(ModelBase):
             alias = loss.__name__
 
         if isinstance(loss, str):
-
-            loss_class = get_loss(loss)
-            alias = loss if loss_class is not None else alias
-            if alias in self._losses:
-                dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
-                alias = alias + '_' + str(len(dup_keys) + 1)
-            self._losses[alias] = loss_class(**kwargs) if len(kwargs) > 0 else loss()
-            if hasattr(loss, 'forward'):
-                argnames = get_signature(self._losses[alias].forward, alias)
+            if loss=='output':
+                self.use_output_as_loss = True
+                return self
             else:
-                argnames = get_signature(self._losses[alias].__call__, alias)
+                loss_class = get_loss(loss)
+                alias = loss if loss_class is not None else alias
+                if alias in self._losses:
+                    dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
+                    alias = alias + '_' + str(len(dup_keys) + 1)
+                self._losses[alias] = loss_class(**kwargs) if len(kwargs) > 0 else loss()
+                if hasattr(loss, 'forward'):
+                    argnames = get_signature(self._losses[alias].forward, alias)
+                else:
+                    argnames = get_signature(self._losses[alias].__call__, alias)
         elif inspect.isclass(loss) and inspect._is_type(loss):
             alias = loss.__class__.__name__ if alias is None or len(alias) == 0 else alias
             if alias in self._losses:
@@ -522,42 +525,43 @@ class Model(ModelBase):
                             Warning(
                                 'input argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
                                     arg))
-
-                    if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
-                        self.training_context['data_feed'] = data_feed
+                    #
+                    # if len(self._signature.inputs.key_list) == 1 and data_feed[self._signature.inputs.key_list[0]] != None:
+                    #     self.training_context['data_feed'] = data_feed
 
                     # check for target
-                    for i in range(len(self.targets)):
-                        arg = self.targets.key_list[i]
-                        data_feed[arg] = ''
-                        if len(train_data) == 1:
-                            data_feed[self.targets.key_list[0]] = train_data.key_list[0]
-                        elif arg in available_fields:
-                            data_feed[arg] = arg
-                            available_fields.remove(arg)
-                        elif arg == 'target' and 'label' in available_fields:
-                            data_feed[arg] = 'label'
-                            available_fields.remove('label')
-                        elif arg == 'target' and len(available_fields) == 1:
-                            data_feed[arg] = available_fields[0]
-                            available_fields.remove(available_fields[0])
-                        elif len(available_fields) > 0:
-                            target_shape = outshapes[i]
-                            for item in available_fields:
-                                data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
-                                if target_shape == data_shape:
-                                    data_feed[arg] = item
-                                    available_fields.remove(item)
-                                elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
-                                        train_data[item].dtype)) and target_shape[:-1] == data_shape:
-                                    data_feed[arg] = item
-                                    available_fields.remove(item)
-                                else:
-                                    Warning(
-                                        'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
-                                            arg))
-                    if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
-                        self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
+                    if len(available_fields)>0:
+                        for i in range(len(self.targets)):
+                            arg = self.targets.key_list[i]
+                            data_feed[arg] = ''
+                            if len(train_data) == 1:
+                                data_feed[self.targets.key_list[0]] = train_data.key_list[0]
+                            elif arg in available_fields:
+                                data_feed[arg] = arg
+                                available_fields.remove(arg)
+                            elif arg == 'target' and 'label' in available_fields:
+                                data_feed[arg] = 'label'
+                                available_fields.remove('label')
+                            elif arg == 'target' and len(available_fields) == 1:
+                                data_feed[arg] = available_fields[0]
+                                available_fields.remove(available_fields[0])
+                            elif len(available_fields) > 0:
+                                target_shape = outshapes[i]
+                                for item in available_fields:
+                                    data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
+                                    if target_shape == data_shape:
+                                        data_feed[arg] = item
+                                        available_fields.remove(item)
+                                    elif ('int64' in str(train_data[item].dtype) or 'int32' in str(
+                                            train_data[item].dtype)) and target_shape[:-1] == data_shape:
+                                        data_feed[arg] = item
+                                        available_fields.remove(item)
+                                    else:
+                                        Warning(
+                                            'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
+                                                arg))
+                        if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
+                            self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
 
                     self.training_context['data_feed'] = data_feed
                     print('data_feed', data_feed)
@@ -838,8 +842,8 @@ class Model(ModelBase):
 
 
 class ImageClassificationModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageClassificationModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageClassificationModel, self).__init__(inputs, input_shape,output)
 
         self._class_names = []
         self.preprocess_flow = []
@@ -905,8 +909,8 @@ class ImageClassificationModel(Model):
 
 
 class ImageDetectionModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageDetectionModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageDetectionModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
         self.detection_threshould = 0.5
 
@@ -939,12 +943,12 @@ class ImageDetectionModel(Model):
 
 
 class ImageSegmentationModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(ImageSegmentationModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(ImageSegmentationModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
 
 
 class LanguageModel(Model):
-    def __init__(self, inputs=None, output=None, input_shape=None):
-        super(LanguageModel, self).__init__(inputs, output, input_shape)
+    def __init__(self, inputs=None,  input_shape=None,output=None):
+        super(LanguageModel, self).__init__(inputs, input_shape,output)
         self.preprocess_flow = []
