@@ -309,7 +309,13 @@ class TrainingPlan(object):
                                 trainingitem.training_context['data_feed'] = data_feed
                                 print('data_feed for {0} :{1}'.format(trainingitem.name,
                                                                       data_feed))  # elif len(outputs)==1
-
+                    elif len(trainingitem.signature.inputs)  == len(data_loader.signature.outputs) :
+                        for inp in trainingitem.signature.inputs.key_list:
+                            if inp in data_loader.signature.outputs.key_list:
+                                data_feed[inp]=inp
+                        if len(data_feed.key_list)==0:
+                            for i in range(len(trainingitem.signature.inputs)):
+                                data_feed[trainingitem.signature.inputs.key_list[i]] = data_loader.signature.outputs.key_list[i]
                     else:
                         raise RuntimeError(
                             'the number of models input plus the numbers of  targets should equal to the numbers of '
